@@ -1,7 +1,6 @@
 #include <string.h>
 #include <stdlib.h>
 #include "everything.h"
-#include <gio/gio.h>
 
 static gboolean abort_on_error = TRUE;
 
@@ -753,7 +752,7 @@ test_array_int_none_out(int *len) {
 
 /**
  * test_array_int_null_in:
- * @arr: (array length=1) (allow-none):
+ * @arr: (array length=len) (allow-none):
  * @len: length
  */
 void test_array_int_null_in (int *arr, int len)
@@ -763,7 +762,7 @@ void test_array_int_null_in (int *arr, int len)
 
 /**
  * test_array_int_null_out:
- * @arr: (out) (array length=1) (allow-none):
+ * @arr: (out) (array length=len) (allow-none):
  * @len: (out) : length
  */
 void test_array_int_null_out (int **arr, int *len)
@@ -1778,6 +1777,24 @@ test_obj_do_matrix (TestObj *obj, const char *somestr)
   return TEST_OBJ_GET_CLASS (obj)->matrix (obj, somestr);
 }
 
+/**
+ * test_object_null_in:
+ * @obj: (allow-none) A #TestObj
+ */
+void
+test_object_null_in (TestObj *obj)
+{
+}
+
+/**
+ * test_object_null_out:
+ * @obj: (allow-none) (out) A #TestObj
+ */
+void
+test_object_null_out (TestObj **obj)
+{
+}
+
 typedef struct _CallbackInfo CallbackInfo;
 
 struct _CallbackInfo
@@ -1832,6 +1849,25 @@ test_callback (TestCallback callback)
     return 0;
 }
 
+/**
+ * test_multi_callback:
+ * @callback: (scope call) (allow-none):
+ *
+ **/
+int
+test_multi_callback (TestCallback callback)
+{
+    int sum = 0;
+    if (callback != NULL) {
+        sum += callback();
+        sum += callback();
+    }
+
+    return sum;
+}
+
+
+
 /**
  * test_simple_callback:
  * @callback: (scope call) (allow-none):
@@ -1960,22 +1996,59 @@ test_callback_thaw_async (void)
   return retval;
 }
 
-/**
- * test_callback_infinite:
- * @callback: (scope infinite):
- *
- * Infinite - callback persists forever.
- **/
-
-static GSList *infinite_callbacks = NULL;
-
-int
-test_callback_infinite (TestCallbackUserData callback,
-                       gpointer user_data)
+void
+test_async_ready_callback (GAsyncReadyCallback callback)
 {
-  infinite_callbacks = g_slist_prepend(infinite_callbacks, callback);
+  GSimpleAsyncResult *result = g_simple_async_result_new (NULL, callback, NULL,
+    test_async_ready_callback);
+  g_simple_async_result_complete_in_idle (result);
+}
 
-  return callback(user_data);
+/**
+ * test_obj_instance_method_callback:
+ * @callback: (scope call) (allow-none):
+ *
+ **/
+void
+test_obj_instance_method_callback (TestObj *obj, TestCallback callback)
+{
+    if (callback != NULL)
+        callback();
+}
+
+/**
+ * test_obj_static_method_callback:
+ * @callback: (scope call) (allow-none):
+ *
+ **/
+void
+test_obj_static_method_callback (TestCallback callback)
+{
+    if (callback != NULL)
+        callback();
+}
+
+/**
+ * test_obj_new_callback:
+ * @callback: (scope notified):
+ *
+ **/
+TestObj *
+test_obj_new_callback (TestCallbackUserData callback, gpointer user_data,
+                       GDestroyNotify notify)
+{
+  CallbackInfo *info;
+
+  callback(user_data);
+
+  info = g_slice_new(CallbackInfo);
+  info->callback = callback;
+  info->notify = notify;
+  info->user_data = user_data;
+
+  notified_callbacks = g_slist_prepend(notified_callbacks, info);
+
+  return g_object_new (TEST_TYPE_OBJ, NULL);
 }
 
 /* interface */
@@ -2107,6 +2180,36 @@ test_wi_802_1x_static_method (int x)
   return 2*x;
 }
 
+/* floating gobject */
+G_DEFINE_TYPE(TestFloating, test_floating, G_TYPE_INITIALLY_UNOWNED);
+
+static void
+test_floating_finalize(GObject *object)
+{
+  g_assert(!g_object_is_floating (object));
+
+  G_OBJECT_CLASS(test_floating_parent_class)->finalize(object);
+}
+
+static void
+test_floating_class_init (TestFloatingClass *klass)
+{
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  gobject_class->finalize = test_floating_finalize;
+}
+
+static void
+test_floating_init (TestFloating *obj)
+{
+}
+
+TestFloating *
+test_floating_new (void)
+{
+  return g_object_new (TEST_TYPE_FLOATING, NULL);
+}
+
+
 /**
  * test_torture_signature_0:
  * @x:
